/* bossmin - an example that emulates data being published into R-GMA by a BOSS 'test' job. */
/*            the R-GMA C++ API is used to create a StreamProducer                          */
/*                                                                                          */
/* Usage: bossmin [iterations] [jobId]                                                      */

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "bossmin.h"

// Copy or full path - don't mix this example with real BOSS:
#include "BossConfig.h"

/* BOSS job ID to pretend to be: */
#define BOSS_JOB_ID "42"

/* Global var - weird fudge to emulate what happens in BOSS */
int iJobId=atoi(BOSS_JOB_ID);
char acHostName[64] ="default";
char *acExecPath;

Boss::Boss()
{

	// Fudge to get start time for dummy
	sprintf(acTStart, "%li", (long int)time(NULL));
	sprintf(acJobId, "%d", iJobId);
	pTimeInt=new edg::info::TimeInterval( (double)10.0);

	// Creates a new producer of bossJobExOutStandardInfo, filled at start and end of run
	// pJobProducer= new edg::info::CircularBufferProducer ("bossJobExOutStandardInfo",
	// "WHERE (bossDatabaseHost=\'" BOSS_DB_HOST "." BOSS_DB_DOMAIN "\' AND bossDatabaseName=\'" BOSS_DB_NAME 
	//	"\' AND bossJobId=\'" + (string)acJobId + "\')",
	//	0);
	// This line sets the buffer size to 10 - which means that 10 rows can be inserted until the buffer starts to be overwritten.
	// pMessageProducer->setMaxBufferSize(1000); not in StreamProducer
	// Above deprecated - using R-GMA 2.x and no longer using Standard Info table


try {
	// Creates a new message producer, into table bossJobExOutMessage
	pMessageProducer= new edg::info::StreamProducer();

} catch (edg::info::RGMAException e) {
	std::cout << "Exception creating producer: " << e.what() << std::endl;
}

try {
 	pMessageProducer->declareTable("bossJobExOutMessage",
		 "WHERE (bossDatabaseHost=\'" BOSS_DB_HOST "." BOSS_DB_DOMAIN "\' AND bossDatabaseName=\'" BOSS_DB_NAME 
		 "\' AND bossJobId=" + (string)acJobId + ")"
		);

} catch (edg::info::RGMAException e) {
	std::cout << "Exception declaring table: " << e.what() << std::endl;
}


try {
	// Set a minimum time period to hold an inserted tuple, even if all current consumers have taken the tuple.
	pTimeInt->setBySeconds( (double)2.0);
	pMessageProducer->setMinRetentionPeriod(*pTimeInt); 
	// Sets the time interval before which the API must have issued an action to the servlet.
	pTimeInt->setBySeconds( (double)120.0);
	pMessageProducer->setTerminationInterval(*pTimeInt);

} catch (edg::info::RGMAException e) {
        std::cout << "Exception tweaking producer: " << e.what() << std::endl;    }

	// Insert E_HOST E_USR E_PATH and T_START into JOB table
	updateJobParameter(iJobId, "JOB", "E_HOST",  acHostName);
	updateJobParameter(iJobId, "JOB", "E_USR",   "ExecUser");
	updateJobParameter(iJobId, "JOB", "E_PATH",  acExecPath);
	updateJobParameter(iJobId, "JOB", "T_START",   acTStart);
}

int Boss::updateJobParameter(int idnum, string tab, string key, string val) 
{
	sprintf(acJobId, "%d", idnum);
	sprintf(acTStamp, "%li", (long int)time(NULL));

	string sUpdateQuery = "INSERT into bossJobExOutMessage VALUES (\'" BOSS_DB_HOST "." BOSS_DB_DOMAIN "\',\'" BOSS_DB_NAME 
		"\',\'" + (string)acJobId + "\',\'" + tab + "\',\'" + key + "\',\'" + val + "\',\'" + (string)acTStamp + "\');";


	//bosscat.info("Entering APIBase constructor");

	// Inserts a row into the message producer
	pMessageProducer->insert(sUpdateQuery);
	return 0;
}

Boss::~Boss(){
	/*cleans up */
	// Insert job end info into the producer
	sprintf(acTStamp, "%li", (long int)time(NULL));

	// fill Standard Info table:
	// sprintf(acJobId, "%d", iJobId);
	// pJobProducer->insert("INSERT into bossJobExOutStandardInfo VALUES (\'" BOSS_DB_HOST "." BOSS_DB_DOMAIN "\',\'" BOSS_DB_NAME 
	//	"\',\'" + (string)acJobId + "\',\'" + (string)acHostName + "\',\'ExecPath\',\'ExecUser\',\'" + (string)acTStart + 
	//	"\',\'" + (string)acTStamp + "\',\'TStat\',\'RetCode\',\'" + (string)acTStamp + "\')");
	// was ExecHost, TStart and TStop - BOSS will supply these 
	// No longer using Standard Info table


	// insert RET_CODE T_STAT T_STOP
	updateJobParameter(iJobId, "JOB", "RET_CODE", "RetCode");
	updateJobParameter(iJobId, "JOB", "T_STAT",   "TStat");
	updateJobParameter(iJobId, "JOB", "T_STOP", acTStamp);

	pMessageProducer->undeclareTable("bossJobExOutMessage");
	delete pMessageProducer;
}


int main(int argc, char** argv) {

  int iNumIter=0;
  if (argc > 1) iNumIter=iNumIter=atoi(argv[1]);
  if (iNumIter < 1) { 
	iNumIter=1;
  }

  std::cout << "bossmin - the BOSS dummy producer\n\nversion " BOSS_VERSION "\n\n";

  if (argc > 2) iJobId=atoi(argv[2]);
  if (iJobId < 1) { 
	iJobId=atoi(BOSS_JOB_ID);
  }


  ifstream fIn("/etc/HOSTNAME");
  if (!fIn) {
	std::cout << "Can't read hostname\n\n";
	// return 1;
  } else {
  	fIn >> acHostName;
  }
  fIn.close();

  if ( gethostname(acHostName, (size_t)64) != 0) {
	std::cout << "Can't get hostname\n\n";
  }

  acExecPath=argv[0];
    
  /*The next couple of lines is used to initialise log4cpp*/
/* Hack out from R-GMA 3.2.22
  string initFileName = "./log4cpp.init";
   try {
    log4cpp::SimpleConfigurator::configure(initFileName);
  } catch(log4cpp::ConfigureFailure& f) {
    std::cout << "Log4cpp Configure Problem " << f.what() << std::endl;
    return -1;
  }
*/

  Boss boss;

  // Start message
  boss.updateJobParameter( iJobId, "test", "comment", "I\'\'ve started...");

  // Loop through some iterations
  char acCount[50];
  for (int iCount=0; iCount<iNumIter; iCount+=1) {
	std::cout << "Waiting 2 seconds" << std::endl;
	system("sleep 2s");

	sprintf(acCount, "%d", iCount);
	boss.updateJobParameter( iJobId, "test", "counter", (string)acCount);
  }

  // End message
  boss.updateJobParameter( iJobId, "test", "comment", "... so I\'\'ll finish.");

  return 0;
}
